@@ -1,5 +1,5 @@
 /* 
- *  Copyright (c) 2008  Noah Snavely (snavely (at) cs.washington.edu)
+ *  Copyright (c) 2008-2010  Noah Snavely (snavely (at) cs.cornell.edu)
  *    and the University of Washington
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,14 +22,20 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <time.h>
 
 #include "BaseApp.h"
+#include "LoadJPEG.h"
 #include "SifterUtil.h"
 
+#include "defines.h"
 #include "horn.h"
 #include "matrix.h"
+#include "qsort.h"
 #include "util.h"
 
+
+
 #define MIN_MATCHES 10
 
 /* Load keys from a file */
@@ -239,47 +245,45 @@ void BaseApp::LoadMatches() {
 
     if (m_match_table != NULL) {
         LoadMatchTable(m_match_table);
-        return;
     } else if (m_match_index_dir != NULL) {
         LoadMatchIndexes(m_match_index_dir);
-        return;
-    }
-
-    printf("[LoadMatches] Loading matches\n");
-
-    int num_images = GetNumImages();
-
-    FILE *f = fopen("match-index.txt", "r");
-    if (f == NULL) {
-        /* Try all pairs */
-        for (int i = 0; i < num_images; i++) {
-            for (int j = i+1; j < num_images; j++) {
-                ReadMatchFile(i, j);
-            }
-        }
     } else {
-        printf("[LoadMatches] Reading matches from 'match-index.txt'\n");
-        fflush(stdout);
+        printf("[LoadMatches] Loading matches\n");
 
-        /* Set all matches to false */
-        // ClearMatches();
-        RemoveAllMatches();
+        int num_images = GetNumImages();
 
-        char buf[256];
-        unsigned int count = 0;
-        while (fgets(buf, 256, f)) {
-            int i1, i2;
-            sscanf(buf, "%d %d\n", &i1, &i2);
-            ReadMatchFile(i1, i2);
-            count++;
+        FILE *f = fopen("match-index.txt", "r");
+        if (f == NULL) {
+            /* Try all pairs */
+            for (int i = 0; i < num_images; i++) {
+                for (int j = i+1; j < num_images; j++) {
+                    ReadMatchFile(i, j);
+                }
+            }
+        } else {
+            printf("[LoadMatches] Reading matches from 'match-index.txt'\n");
+            fflush(stdout);
+
+            /* Set all matches to false */
+            // ClearMatches();
+            RemoveAllMatches();
+
+            char buf[256];
+            unsigned int count = 0;
+            while (fgets(buf, 256, f)) {
+                int i1, i2;
+                sscanf(buf, "%d %d\n", &i1, &i2);
+                ReadMatchFile(i1, i2);
+                count++;
+            }
+
+            printf("[LoadMatches] Read %d match files\n", count);
+            fflush(stdout);
+
+            fclose(f);
         }
-
-        printf("[LoadMatches] Read %d match files\n", count);
-        fflush(stdout);
-
-        fclose(f);
     }
-    
+
     // WriteMatchTableDrew(".init");
 
     PruneDoubleMatches();
@@ -328,7 +332,7 @@ void BaseApp::LoadImageNamesFromFile(FILE *f)
             data.m_key_name = strdup(key_buf);
         }
 
-        printf("Keyname: %s\n", data.m_key_name);
+        // printf("Keyname: %s\n", data.m_key_name);
 
 #if 0
 	if (log != NULL) {
@@ -386,7 +390,7 @@ void BaseApp::LoadImageNamesFromFile(FILE *f)
 	ImageData data;
 
 	// m_imgs.push_back(img);
-	printf("Adding image %s\n", toks[0].c_str());
+	// printf("Adding image %s\n", toks[0].c_str());
         char buf[512];
         sprintf(buf, "%s/%s", m_image_directory, toks[0].c_str());
 
@@ -494,6 +498,16 @@ void BaseApp::ReadBundleFile(char *filename)
         /* Translation */
         fscanf(f, "%lf %lf %lf\n", t+0, t+1, t+2);
 
+#if 0
+        if (m_bundle_version < 0.3) {
+            R[2] = -R[2];
+            R[5] = -R[5];
+            R[6] = -R[6];
+            R[7] = -R[7];
+            t[2] = -t[2];
+        }
+#endif
+
         if (focal_length <= 100.0 || m_image_data[i].m_ignore_in_bundle) {
             /* No (or bad) information about this camera */
             m_image_data[i].m_camera.m_adjusted = false;
@@ -527,6 +541,9 @@ void BaseApp::ReadBundleFile(char *filename)
         fscanf(f, "%lf %lf %lf\n", 
             pt.m_pos + 0, pt.m_pos + 1, pt.m_pos + 2);
 
+        // if (m_bundle_version < 0.3)
+        //     pt.m_pos[2] = -pt.m_pos[2];
+
         /* Color */
         fscanf(f, "%f %f %f\n", 
             pt.m_color + 0, pt.m_color + 1, pt.m_color + 2);
@@ -547,32 +564,44 @@ void BaseApp::ReadBundleFile(char *filename)
                 //        "Removing view %d from point %d\n", view, i);
             } else {
                 /* Check cheirality */
-                // double Rp[3], p[3];
-                // matrix_product(3, 3, 3, 1, 
-                //             m_image_data[view].m_camera.m_R, pt.m_pos, Rp);
-                // matrix_sum(3, 1, 3, 1, Rp, m_image_data[view].m_camera.m_t, p);
                 bool val = (m_bundle_version >= 0.3);
+
                 double proj_test[2];
                 if (m_image_data[view].m_camera.
                     Project(pt.m_pos, proj_test) == val) {
-
+                    
                     pt.m_views.push_back(ImageKey(view, key));
                 } else {
                     printf("[SifterApp::ReadBundleFile] "
                            "Removing view %d from point %d [cheirality]\n", 
-                           view, i);
+                               view, i);
+                    // pt.m_views.push_back(ImageKey(view, key));
                 }   
             }
             // pt.m_views.push_back(ImageKey(view, key));
-
+            
             if (m_bundle_version >= 0.3) {
                 double x, y;
                 fscanf(f, "%lf %lf", &x, &y);
             }
         }
 
-        /* Add the point to the list */
-        // m_point_data[i] = pt;
+        // #define CROP_POINT_CLOUD
+#ifdef CROP_POINT_CLOUD
+        const double x_min = 1.327;
+        const double x_max = 3.556;
+        const double y_min = -1.414;
+        const double y_max = 1.074;
+        const double z_min = -5.502;
+        const double z_max = -3.288;
+        
+        if (pt.m_pos[0] < x_min || pt.m_pos[0] > x_max ||
+            pt.m_pos[1] < y_min || pt.m_pos[1] > y_max ||
+            pt.m_pos[2] < z_min || pt.m_pos[2] > z_max) {
+         
+            pt.m_views.clear();
+        }
+#endif /* CROP_POINT_CLOUD */
     }
 
 #if 0    
@@ -724,6 +753,13 @@ void BaseApp::DumpOutputFile(char *output_dir, char *filename,
 {
     clock_t start = clock();
 
+    int num_visible_points = 0;
+    
+    for (int i = 0; i < num_points; i++) {
+        if (pt_views[i].size() > 0)
+            num_visible_points++;
+    }
+
     char buf[256];
     sprintf(buf, "%s/%s", output_dir, filename);
 
@@ -739,7 +775,7 @@ void BaseApp::DumpOutputFile(char *output_dir, char *filename,
     fprintf(f, "# Bundle file v0.3\n");
     // }
 
-    fprintf(f, "%d %d\n", num_images, num_points);
+    fprintf(f, "%d %d\n", num_images, num_visible_points);
 
     /* Dump cameras */
     for (int i = 0; i < num_images; i++) {
@@ -794,30 +830,35 @@ void BaseApp::DumpOutputFile(char *output_dir, char *filename,
 
     /* Dump points */
     for (int i = 0; i < num_points; i++) {
-        /* Position */
-        fprintf(f, "%0.10e %0.10e %0.10e\n", 
-                Vx(points[i]), Vy(points[i]), Vz(points[i]));
-        // Vx(points[idx]), Vy(points[idx]), Vz(points[idx]));
-
-        /* Color */
-        fprintf(f, "%d %d %d\n", 
-                iround(Vx(colors[i])), 
-                iround(Vy(colors[i])), 
-                iround(Vz(colors[i])));
-
         int num_visible = (int) pt_views[i].size();
-        fprintf(f, "%d", num_visible);
-        for (int j = 0; j < num_visible; j++) {
-            int img = added_order[pt_views[i][j].first];
-            int key = pt_views[i][j].second;
 
-            double x = m_image_data[img].m_keys[key].m_x;
-            double y = m_image_data[img].m_keys[key].m_y;
+        if (num_visible > 0) {
 
-            fprintf(f, " %d %d %0.4f %0.4f", img, key, x, y);
+            /* Position */
+            fprintf(f, "%0.10e %0.10e %0.10e\n", 
+                    Vx(points[i]), Vy(points[i]), Vz(points[i]));
+            // Vx(points[idx]), Vy(points[idx]), Vz(points[idx]));
+
+            /* Color */
+            fprintf(f, "%d %d %d\n", 
+                    iround(Vx(colors[i])), 
+                    iround(Vy(colors[i])), 
+                    iround(Vz(colors[i])));
+
+            int num_visible = (int) pt_views[i].size();
+            fprintf(f, "%d", num_visible);
+            for (int j = 0; j < num_visible; j++) {
+                int img = added_order[pt_views[i][j].first];
+                int key = pt_views[i][j].second;
+                
+                double x = m_image_data[img].m_keys[key].m_x;
+                double y = m_image_data[img].m_keys[key].m_y;
+                
+                fprintf(f, " %d %d %0.4f %0.4f", img, key, x, y);
+            }
+            
+            fprintf(f, "\n");
         }
-
-        fprintf(f, "\n");
     }
 
 #if 0
@@ -876,15 +917,16 @@ void BaseApp::WriteCamerasXML(const char *filename)
     
     fprintf(f, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n\n");
     char *url_base = 
-	"http://grail.cs.washington.edu/projects/phototour/trevi/images";
+	"http://grail.cs.cornell.edu/projects/phototour/trevi/images";
 
     fprintf(f, "<url_base> %s </url_base>\n", url_base);
     fprintf(f, "<cameras>\n");
     
     int num_images = (int) m_image_data.size();
     for (int i = 0; i < num_images; i++) {
-        if (m_image_data[i].m_camera.m_adjusted /*&& m_image_data[i].m_licensed*/)
+        if (m_image_data[i].m_camera.m_adjusted /*&& m_image_data[i].m_licensed*/) {
             m_image_data[i].WriteCameraXML(f);
+        }
     }
 
     fprintf(f, "</cameras>\n");
@@ -925,6 +967,39 @@ void BaseApp::WritePointsXML(const char *filename)
 	   num_ge2, num_points, min_views);
 }
 
+/* Point I/O */
+void BaseApp::WritePointsGeoXML(const char *filename) 
+{
+    FILE *f = fopen(filename, "w");
+    int min_views = 2;
+
+    if (f == NULL) {
+	printf("[SifterApp::WritePointsXML] "
+	       "Error opening file %s for writing\n", filename);
+	return;
+    }
+    
+    fprintf(f, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n\n");
+    fprintf(f, "<points>\n");
+    
+    int num_points = (int) m_point_data.size();
+    int num_ge2 = 0;
+
+    for (int i = 0; i < num_points; i++) {
+        // if (m_num_views_orig[i] >= min_views) {
+        if ((int) m_point_data[i].m_views.size() >= min_views) {
+            m_point_data[i].WriteGeoXML(f);
+            num_ge2++;
+        }
+    }
+
+    fprintf(f, "</points>\n");
+    fclose(f);
+
+    printf("[SifterApp::WritePointsXML] %d / %d points seen by >= %d views\n",
+	   num_ge2, num_points, min_views);
+}
+
 void BaseApp::ReadMatchTableDrew(const char *append) 
 {
     int num_images = GetNumImages();
@@ -1049,6 +1124,8 @@ void BaseApp::WriteMatchTableDrew(const char *append)
 static char ply_header[] = 
 "ply\n"
 "format ascii 1.0\n"
+"element face 0\n"
+"property list uchar int vertex_indices\n"
 "element vertex %d\n"
 "property float x\n"
 "property float y\n"
@@ -1365,3 +1442,13 @@ void BaseApp::InitializeImagesFromFile(FILE *f)
 
     UnscaleCameras(num_images);
 }
+
+void BaseApp::ReadLines3D(char *filename) 
+{
+
+}
+
+void BaseApp::WriteLines3D(char *filename)
+{
+
+}
