@@ -1,5 +1,5 @@
 /* 
- *  Copyright (c) 2008  Noah Snavely (snavely (at) cs.washington.edu)
+ *  Copyright (c) 2008-2010  Noah Snavely (snavely (at) cs.cornell.edu)
  *    and the University of Washington
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
 
 /* BundlerApp.cpp */
 
-#ifndef WIN32_MVC
+#ifndef WIN32
 #include <getopt.h>
 #else
 #include "getopt.h"
@@ -28,7 +28,7 @@
 #include <string.h>
 #include <time.h>
 
-#ifdef WIN32_MVC
+#ifdef WIN32
 #include <conio.h>
 #endif
 
@@ -100,7 +100,7 @@ static void ReadFisheyeParameters(char *filename,
 	//     toks.push_back(tok);
 	// }
 
-	if (toks[0] == "FisheyeCenter:") {
+	if (strcmp(toks[0].c_str(), "FisheyeCenter:") == 0) {
 	    if (toks.size() != 3) {
 		printf("FisheyeCenter needs two arguments!\n");
 		exit(1);
@@ -108,21 +108,21 @@ static void ReadFisheyeParameters(char *filename,
 		fCx = atof(toks[1].c_str());
 		fCy = atof(toks[2].c_str());
 	    }
-	} else if (toks[0] == "FisheyeRadius:") {
+	} else if (strcmp(toks[0].c_str(), "FisheyeRadius:") == 0) {
 	    if (toks.size() != 2) {
 		printf("FisheyeRadius needs one argument!\n");
 		exit(1);
 	    } else {
 		fRad = atof(toks[1].c_str());
 	    }
-	} else if (toks[0] == "FisheyeAngle:") {
+	} else if (strcmp(toks[0].c_str(), "FisheyeAngle:") == 0) {
 	    if (toks.size() != 2) {
 		printf("FisheyeAngle needs one argument!\n");
 		exit(1);
 	    } else {
 		fAngle = atof(toks[1].c_str());
 	    }	    
-	} else if (toks[0] == "FisheyeFocal:") {
+	} else if (strcmp(toks[0].c_str(), "FisheyeFocal:") == 0) {
 	    if (toks.size() != 2) {
 		printf("FisheyeFocal needs one argument!\n");
 		exit(1);
@@ -226,8 +226,6 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
 {
     /* Read options */
     while (1) {
-
-#ifndef WIN32_MVC
 	static struct option long_options[] = {
 	    {"fisheye",      1, 0, 'f'},
             {"intrinsics",   1, 0, 357}, //
@@ -253,6 +251,7 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
             {"rotate_cameras", 1, 0, 309},//
             {"zero_distortion_params",0, 0, 363},//
             {"enrich_points", 0, 0, 361},//
+            {"output_relposes", 1, 0, 368},
 
 
             {"compute_color_statistics", 0, 0, 323},//
@@ -283,14 +282,15 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
             {"trust_focal_estimate", 0, 0, '_'},
             {"estimate_ignored", 0, 0, 344},//
             {"reposition_scene", 0, 0, 'R'},//
-            {"segment_sky", 0, 0, 350},//
+            {"segment_sky", 1, 0, 350},//
             {"keypoint_border_width", 1, 0, 355},//
             {"keypoint_border_bottom", 1, 0, 365},//
 
 	    {"min_track_views", 1, 0, 'V'},//
 	    {"max_track_views", 1, 0, 320},//
             {"min_max_matches", 1, 0, 362},//
-            
+            {"min_feature_matches", 1, 0, 369},//
+
 	    {"ray_angle_threshold", 1, 0, 'N'},
 	    {"estimate_distortion", 0, 0, 347},
             {"distortion_weight", 1, 0, 348},//
@@ -307,6 +307,8 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
             {"max_proj_error_threshold", 1, 0, 318},
             {"use_angular_score", 0, 0, 349},//
             
+	    {"up_image",             1, 0, 'E'},
+	    {"estimate_up_vector_szeliski", 0, 0, 'Z'},
 	    {"min_camera_distance_ratio", 1, 0, 'C'},//
 	    // {"baseline_threshold",   1, 0, 'B'},
             {"optimize_for_fisheye",  0, 0, 'B'},//
@@ -349,20 +351,10 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
 
 	    {0, 0, 0, 0}
 	};
-#endif
 
-	// printf("optind[pre] = %d\n", optind);
-
-#ifdef WIN32_MVC
-    int c = getopt(argc, argv, "cf:d:o:a:i:xb:Ae:z:Q:g:ZT:E:L:!:5:67:8:9:0:#:@:lG:R");
-    printf("  option: %c\n", c);
-#else
 	int option_index;
 	int c = getopt_long(argc, argv, "f:do:a:i:x",
 			    long_options, &option_index);
-#endif
-
-	// printf("optind[post] = %d\n", optind);
 
 	if (c == -1)
 	    break;
@@ -456,6 +448,16 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
 
 
 
+            case 368:
+                m_output_relposes = true;
+                m_output_relposes_file = strdup(optarg);
+                break;
+
+            case 350:
+                m_segment_sky = true;
+                m_sky_model_file = strdup(optarg);
+                break;
+
             case 305:
                 m_scale_focal = atof(optarg);
                 break;
@@ -602,13 +604,17 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
 		m_min_track_views = atoi(optarg);
 		break;
 
+	    case 320:
+		m_max_track_views = atoi(optarg);
+		break;
+
             case 362:
                 m_min_max_matches = atoi(optarg);
                 break;
 
-	    case 320:
-		m_max_track_views = atoi(optarg);
-		break;
+            case 369:
+                m_min_num_feat_matches = atoi(optarg);
+                break;
 
 	    case 'N':
 		m_ray_angle_threshold = atof(optarg);
@@ -763,7 +769,11 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
 
 		optind = optind_curr;
 
+#ifdef WIN32
+        return;
+#else
 		break;
+#endif
 	    }
 
 	    default:
@@ -774,7 +784,7 @@ void BundlerApp::ProcessOptions(int argc, char **argv)
 }
 
 
-#ifdef WIN32_MVC
+#ifdef WIN32
 static void sifter_die() {
     getch();
 }
@@ -827,10 +837,11 @@ bool BundlerApp::OnInit()
         exit(1);
     }
 
-#ifdef _WIN32
-	if (m_get_key_on_exit)
-		atexit(sifter_die);
-#endif
+    if (m_fixed_focal_length && m_estimate_distortion) {
+        printf("Error: --fixed_focal_length and --estimate_distortion "
+               "are currently incompatible\n");
+        exit(1);
+    }
 
     printf("[BundlerApp::OnInit] Loading frame...\n");
 
@@ -846,15 +857,6 @@ bool BundlerApp::OnInit()
     printf("tested frame\n");
 #endif
 
-
-#ifndef __NO_UI__
-    SifterFrame *frame = NULL;
-    frame = new SifterFrame();
-    printf("[BundlerApp::OnInit] Showing frame...\n");
-    frame->Show(TRUE);
-    wxCommandEvent dummy = wxCommandEvent();
-#endif
-
     printf("[BundlerApp::OnInit] Loading images...\n");
     fflush(stdout);
     if (load_file) {
@@ -959,23 +961,24 @@ bool BundlerApp::OnInit()
         printf("[BundlerApp::OnInit] Reading bundle file...\n");
         ReadBundleFile(m_bundle_file);
 
+        if (m_bundle_version < 0.3) {
+            printf("[BundlerApp::OnInit] Reflecting scene...\n");
+            FixReflectionBug();
+        }
+
+        // AutoTagImages();
+
 #ifndef __DEMO__
 
 #endif /* __DEMO__ */
 
         if (m_compress_list) {
-            if (m_bundle_version < 0.3)
-                FixReflectionBug();
-
             // ParseCommand("UndistortAll", NULL);
             OutputCompressed();
             return 0;
         }
 
         if (m_reposition_scene) {
-            if (m_bundle_version < 0.3)
-                FixReflectionBug();
-            
             double center[3], R[9], scale;
             RepositionScene(center, R, scale);
             OutputCompressed("reposition");
@@ -1016,10 +1019,6 @@ bool BundlerApp::OnInit()
 
 #endif /* __DEMO__ */
 
-        if (m_bundle_version < 0.3) {
-            printf("[BundlerApp::OnInit] Reflecting scene...\n");
-            FixReflectionBug();
-        }
         
 #ifndef __DEMO__
 
@@ -1033,6 +1032,20 @@ bool BundlerApp::OnInit()
 
 
         
+        if (m_output_relposes) {
+            double center[3], R[9], scale;
+            RepositionScene(center, R, scale);
+            RepositionScene(center, R, scale);
+            // OutputRelativePoses2D(m_output_relposes_file);
+            OutputRelativePoses3D(m_output_relposes_file);
+            return 0;
+        }
+
+        if (m_compute_covariance) {
+            ComputeCameraCovariance();
+            return 0;
+        }
+        
 #define MIN_POINT_VIEWS 3 // 0 // 2
         if (!m_run_bundle) {
             SetMatchesFromPoints(MIN_POINT_VIEWS);
@@ -1054,6 +1067,9 @@ bool BundlerApp::OnInit()
             printf("[BundlerApp::OnInit] Computing camera orientations...\n");
             ComputeImageRotations();
 
+            // printf("[BundlerApp::OnInit] Computing ground plane...\n");
+            // FindGroundPlane();
+
             double center[3], R[9], scale;
             RepositionScene(center, R, scale);
 
@@ -1061,13 +1077,6 @@ bool BundlerApp::OnInit()
             if (m_rerun_bundle) {
                 ReRunSFM();
             }
-
-#if 0
-            if (m_compute_covariance) {
-                ComputeCameraCovariance();
-                exit(0);
-            }
-#endif
 #endif
         }
 
@@ -1122,11 +1131,6 @@ bool BundlerApp::OnInit()
     return true;
 }
 
-
-int g_datalist_unset_flags;
-int g_datalist_set_flags;
-
-#ifdef __NO_UI__
 static BundlerApp *bundler_app = NULL;
 
 BundlerApp &wxGetApp() {
@@ -1143,4 +1147,3 @@ int main(int argc, char **argv)
 
     bundler_app->OnInit();
 }
-#endif
