@@ -1,5 +1,5 @@
 /* 
- *  Copyright (c) 2008  Noah Snavely (snavely (at) cs.washington.edu)
+ *  Copyright (c) 2008-2010  Noah Snavely (snavely (at) cs.cornell.edu)
  *    and the University of Washington
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -808,6 +808,25 @@ void CameraInfo::GetViewDirection(double *view) const
     view[2] = -m_R[8];
 }
 
+/* Get the twist angle of the camera */
+double CameraInfo::GetTwistAngleRadians() const
+{
+    double R[9];
+    GetPose(R);
+
+    double c_twist = 
+        (R[0] * R[8] - R[6] * R[2]) / sqrt(1 - R[5] * R[5]);
+    
+    c_twist = CLAMP(c_twist, -1.0 + 1.0e-8, 1.0 - 1.0e-8);
+
+    double angle = acos(c_twist);  
+
+    if (R[3] < 0.0)
+        return -angle;
+    else 
+        return angle;
+}
+
 /* Return the halfspace in front of the camera */
 void CameraInfo::GetFrontHalfspace(double *plane) const
 {
@@ -928,9 +947,13 @@ void CameraInfo::WriteLinks(FILE *f)
 
 void CameraInfo::WriteFile(FILE *f)
 {
-	double temp[3];
-	GetPosition(temp);
-	fprintf(f,"%f %f %f\n",temp[0],temp[1],temp[2]);
+    if (m_adjusted) {
+        double temp[3];
+        GetPosition(temp);
+        fprintf(f,"%f %f %f\n",temp[0],temp[1],temp[2]);
+    } else {
+        fprintf(f, "0.0 0.0 0.0\n");
+    }
 }
 
 void CameraInfo::WriteXML(FILE *f)
