@@ -1,5 +1,5 @@
 /* 
- *  Copyright (c) 2008  Noah Snavely (snavely (at) cs.washington.edu)
+ *  Copyright (c) 2008-2010  Noah Snavely (snavely (at) cs.cornell.edu)
  *    and the University of Washington
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "ImageData.h"
 #include "SifterUtil.h"
@@ -41,11 +42,7 @@
 #include "transform.h"
 #include "util.h"
 
-#ifndef WIN32_MVC
 #include "LoadJPEG.h"
-#else
-#include "FastJPEG.h"
-#endif
 
 #define SUBSAMPLE_LEVEL 1
 
@@ -223,7 +220,7 @@ void ImageData::InitFromString(char *buf, char *path, bool fisheye_by_default)
 
     ImageData data;
 
-    printf("[ImageData::InitFromFile] Adding image %s\n", toks[0].c_str());
+    // printf("[ImageData::InitFromFile] Adding image %s\n", toks[0].c_str());
 
     if (path == NULL || strcmp(path, ".") == 0 || toks[0].c_str()[0] == '/') {
         m_name = strdup(toks[0].c_str());
@@ -743,7 +740,7 @@ void ImageData::ExtractFeatures(char *sift_binary, bool undistort)
 #ifndef __DEMO__
     /* Find the extension */
     const char *in = m_name;
-    char *ext = rindex(in, '.');
+    char *ext = strrchr((char *) in, '.');
     char out[256];
 	
     strncpy(out, in, ext - in);
@@ -835,8 +832,8 @@ void ImageData::LoadKeys(bool descriptor, bool undistort) {
         
         /* Now make the image center the origin */
         for (int k = 0; k < (int) kps.size(); k++) {
-            kps[k].m_x -= 0.5 * GetWidth();
-            kps[k].m_y -= 0.5 * GetHeight();
+            kps[k].m_x -= 0.5 * (GetWidth() - 1);
+            kps[k].m_y -= 0.5 * (GetHeight() - 1);
         }
         
         m_keys = kps;
@@ -881,6 +878,7 @@ void ImageData::UnloadKeys() {
         }
 
         m_keys_desc.clear();
+        m_keys_desc_loaded = false;
     } else if (m_keys_loaded) {
         m_keys.clear();
         m_keys_loaded = false;
@@ -987,9 +985,9 @@ void ImageData::CacheDimensions()
             }
 	}
     }
-	
+
     m_width = w;
-	m_height = h;
+    m_height = h;
     m_cached_dimensions = true;
 }
 
